# 二板一字是否可以买？
# 一字涨停卖点

import sys
import os
import numpy as np
import datetime
import talib
from QuadQuanta.data.update_data import update_day_bar
from QuadQuanta.core.strategy import BaseStrategy

# from QuadQuanta.data.clickhouse_api import query_clickhouse, query_limit_count, query_N_clickhouse
from QuadQuanta.data.mongodb_api import insert_mongodb, save_mongodb, query_mongodb

# from QuadQuanta.portfolio import Account
from QuadQuanta.data.get_data import get_trade_days, get_securities_info, get_bars
from QuadQuanta.utils.logs import logger

FILENAME = os.path.basename(sys.argv[0]).split(".py")[0]
# 振幅
Amplitude = 6


class LineLimit(BaseStrategy):
    def __init__(
            self, code=None, start_date=None, end_date=None, frequency="day", **kwargs
    ):
        super().__init__(code, start_date, end_date, frequency, **kwargs)
        self.pre_volume_dict = {}
        self.symbol_list = []
        self.res_symbol = []
        self.success_symbol = []
        self.fail_symbol = []
        self.hold_symbol = []

        self.start_date = start_date

    def init(self):
        pass

    def pick_symbol(self, bar):
        """
        初选标，今日一字，涨幅大于6
        Parameters
        ----------
        bar

        Returns
        -------

        """
        code = bar["code"]
        high = round(bar["high"], 2)
        open_ = round(bar["open"], 2)
        high_limit = round(bar["high_limit"], 2)
        close = round(bar["close"], 2)
        pre_close = round(bar["pre_close"], 2)
        high_rate = 100 * (high - pre_close) / pre_close
        close_rate = 100 * (close - pre_close) / pre_close
        # open_rate = 100 * (open_ - pre_close) / pre_close
        amount = bar["amount"]

        if amount > pow(10, 8):
            if open_ == high_limit and high_rate > 6:
                self.symbol_list.append(code)

    def on_day_bar(self, bars: np.ndarray):
        """
        成交量大于昨日两倍，最高点大于最近20日高点（突破股）,20个交易日振幅在20%内
        Parameters
        ----------
        bars

        Returns
        -------

        """
        today_bar = bars[-1]
        code = today_bar["code"]
        pre_bar = bars[-2]
        pre_high_limit = round(pre_bar["high_limit"], 2)
        # today_volume = today_bar["volume"]
        # pre_volume = pre_bar["volume"]
        # pre_high = round(pre_bar['high'])
        max_20_high = max(bars[20:-1]["close"])
        min_20_low = min(bars[20:-1]["close"])
        max_60_high = max(bars[:-1]["high"])

        today_high = round(today_bar["high"], 2)
        pre_close = round(today_bar["pre_close"], 2)
        pre_two_close = round(pre_bar["pre_close"], 2)
        pre_close_rate = 100 * (pre_close - pre_two_close) / pre_two_close
        today_high_rate = 100 * (today_high - pre_close) / pre_close
        # print(bars['close'][-11:])
        # MA60 = talib.SMA(bars['close'], 60)[60:]
        # # 30收盘价价序列
        # close_60 = bars['close'][60:]

        if today_high > max_60_high * 0.97 and pre_close_rate < 9.9:
            # MA_60 = talib.SMA(bars['close'], 60)[60:]
            # # 30收盘价价序列
            # close_60 = bars['close'][60:]
            # lte_count = 0
            # for ma60,close60 in zip(MA_60,close_60):
            #     if ma60 > close60:
            #         lte_count += 1
            # if lte_count < 3:
            self.res_symbol.append(code)

    def next_day_backtest(self, bars):
        """
        次N日回测
        Returns
        -------

        """
        pre_bar = bars[0]
        pre_close = round(pre_bar['close'], 2)
        pre_open = round(pre_bar['open'], 2)

        pre_high_limit = round(pre_bar['high_limit'], 2)
        final_bar = bars[-1]
        symbol = final_bar['code']
        final_close = round(final_bar['close'], 2)
        final_high = round(final_bar['high'], 2)
        final_open = round(final_bar['open'], 2)
        final_low = round(final_bar['low'], 2)
        final_high_limit = round(final_bar['high_limit'], 2)
        # high_seq = bars['high']
        # # pre_close = round(bar['pre_close'], 2)
        start_bar = bars[0]
        start_close = round(start_bar['pre_close'], 2)
        high_rate = 100 * (final_high - start_close) / start_close
        # close_rate = 100 * (final_close - start_close) / start_close
        # high_rate_seq = [100 * (x - start_close) / start_close for x in high_seq]
        # 一字前一日实体阳线涨停
        if pre_close == pre_high_limit and pre_open < 0.95 * pre_close:
            self.res_symbol.append(symbol)



    def syn_backtest(self):
        self.total_assert = []
        next_N_day = 1
        for i in range(0, len(self.trading_date) - next_N_day):
            try:
                date = self.trading_date[i]

                next_date = self.trading_date[i + next_N_day]
                until_trade_days = get_trade_days(
                    start_time="2018-01-01", end_time=date
                )
                self.today_data = self.day_data[self.day_data["date"] == date]
                self.code_list = np.unique(self.today_data["code"])

                for bar in self.today_data:
                    if str(bar["code"]).startswith("688") or str(bar["code"]).startswith("30"):
                        continue
                    self.pick_symbol(bar)
                # history_N_data = get_bars(self.symbol_list, end_time=date, count=51)
                #
                # for symbol in self.symbol_list:
                #     if str(symbol).startswith('30'):
                #         continue
                #     bars = history_N_data[history_N_data["code"] == symbol]
                #     if len(bars) == 51:
                #         self.on_day_bar(bars)
                history_Next_data = get_bars(self.symbol_list, end_time=next_date, count=3)
                for symbol in self.symbol_list:
                    if str(symbol).startswith('30'):
                        continue
                    bars = history_Next_data[history_Next_data["code"] == symbol]
                    if len(bars) == 3:
                        self.next_day_backtest(bars)
                if len(self.res_symbol) > 0:
                    logger.info(f"日期{date}")
                    logger.info(f"{self.res_symbol}")
                    # logger.info(f"{self.success_symbol}")
                    # logger.info(f"一字炸板 {(self.fail_symbol)}")
                    # logger.info(f"回测标数量 {len(self.success_symbol)/len(self.res_symbol)}")

                self.res_symbol = []
                self.success_symbol = []
                self.hold_symbol = []
                self.fail_symbol = []
                self.symbol_list = []

            except Exception as e:
                logger.warning(e)
                continue


def daiyl_n_limit():
    # try:
    #     update_day_bar()
    # except Exception as e:
    #     print(e)
    logger.info(f"filename:{FILENAME}.py, N字板")

    today = datetime.date.today()
    end_time = str(today)
    trade_days_until_today = get_trade_days(start_time="2014-01-01", end_time=end_time)
    start_time = trade_days_until_today[-2]
    # test = DailyNLimit(start_date=start_time,
    #                    end_date=end_time,
    #                    frequency='daily', solid=True)
    test = LineLimit(
        start_date="2023-01-01", end_date="2023-04-01", frequency="daily", solid=True
    )
    test.syn_backtest()


if __name__ == "__main__":
    daiyl_n_limit()
